@@ -106,37 +106,48 @@ pub fn page(
                     }
                     div.toolbar {
                         @if conf.tar_enabled || conf.tar_gz_enabled || conf.zip_enabled {
-                            div.download {
-                                @for archive_method in ArchiveMethod::iter() {
-                                    @if archive_method.is_enabled(conf.tar_enabled, conf.tar_gz_enabled, conf.zip_enabled) {
-                                        (archive_button(archive_method, sort_method, sort_order))
+                            div.tool_row.download_tools {
+                                div.tool data-tool="download" {
+                                    @for archive_method in ArchiveMethod::iter() {
+                                        @if archive_method.is_enabled(conf.tar_enabled, conf.tar_gz_enabled, conf.zip_enabled) {
+                                            (archive_button(archive_method, sort_method, sort_order))
+                                        }
                                     }
                                 }
                             }
                         }
-                        div.toolbar_box_group {
+
+                        div.tool_row.upload_tools {
                             @if conf.file_upload && upload_allowed {
-                                div.toolbar_box {
-                                    form id="file_submit" action=(upload_action) method="POST" enctype="multipart/form-data" {
-                                        p { "Select a file to upload or drag it anywhere into the window" }
-                                        div {
-                                            @match &conf.uploadable_media_type {
-                                                Some(accept) => {input #file-input accept=(accept) type="file" name="file_to_upload" required="" multiple {}},
-                                                None => {input #file-input type="file" name="file_to_upload" required="" multiple {}}
-                                            }
-                                            button type="submit" { "Upload file" }
+                                form.tool id="file_submit" data-tool="upload" action=(upload_action) method="POST" enctype="multipart/form-data" {
+                                    p { "Select a file to upload or drag it anywhere into the window" }
+                                    div {
+                                        @match &conf.uploadable_media_type {
+                                            Some(accept) => {input #file-input accept=(accept) type="file" name="file_to_upload" required="" multiple {}},
+                                            None => {input #file-input type="file" name="file_to_upload" required="" multiple {}}
                                         }
+                                        button type="submit" title="Upload File" { "Upload file" }
                                     }
                                 }
                             }
                             @if conf.mkdir_enabled && upload_allowed {
-                                div.toolbar_box {
-                                    form id="mkdir" action=(mkdir_action) method="POST" enctype="multipart/form-data" {
-                                        p { "Specify a directory name to create" }
-                                        div.toolbar_box {
-                                            input type="text" name="mkdir" required="" placeholder="Directory name" {}
-                                            button type="submit" { "Create directory" }
-                                        }
+                                form.tool id="mkdir" data-tool="mkdir" action=(mkdir_action) method="POST" enctype="multipart/form-data" {
+                                    p { "Specify a directory name to create" }
+                                    div {
+                                        input type="text" name="mkdir" required="" placeholder="Directory name" {}
+                                        button type="submit" title="Create directory" { "Create directory" }
+                                    }
+                                }
+                            }
+                            @if conf.pastebin_enabled && upload_allowed {
+                                form.tool id="pastebin" data-tool="pastebin" {
+                                    p { "Create a text file in the current directory, a random filename will be generated, or you may specify one." }
+                                    div {
+                                        textarea #pastebin_content name="paste_content" title="Text content" required="" { }
+                                    }
+                                    div {
+                                        input type="text" name="paste_filename" title="Filename" placeholder="Filename (Optional)" autocomplete="off" {}
+                                        button type="submit" title="Create file" { "Create file" }
                                     }
                                 }
                             }
@@ -165,7 +176,7 @@ pub fn page(
                                 }
                             }
                             @for entry in entries {
-                                (entry_row(entry, sort_method, sort_order, false, conf.show_exact_bytes, actions_conf))
+                                (entry_row(entry, sort_method, sort_order, false, conf.show_exact_bytes, actions_conf, &conf.route_prefix))
                             }
                         }
                     }
@@ -254,7 +265,7 @@ pub fn raw(entries: Vec<Entry>, is_root: bool, conf: &MiniserveConfig) -> Markup
                             }
                         }
                         @for entry in entries {
-                            (entry_row(entry, None, None, true, conf.show_exact_bytes, None))
+                            (entry_row(entry, None, None, true, conf.show_exact_bytes, None, &conf.route_prefix))
                         }
                     }
                 }
@@ -548,8 +559,10 @@ fn sortable_title(
 }
 
 /// Partial: rm form
-fn rm_form(rm_route: &str, encoded_path: &str) -> Markup {
-    let rm_action = format!("{rm_route}?path={encoded_path}");
+fn rm_form(rm_route: &str, encoded_path: &str, prefix: &str) -> Markup {
+    let stripped_path = encoded_path.strip_prefix(prefix).unwrap_or(encoded_path);
+    let rm_action = format!("{rm_route}?path={stripped_path}");
+
     html! {
         form class="rm_form" action=(rm_action) method="POST" {
             button type="submit" title="Delete" { "✗" }
@@ -571,6 +584,7 @@ fn entry_row(
     raw: bool,
     show_exact_bytes: bool,
     actions_conf: Option<ActionsConf>,
+    route_prefix: &str,
 ) -> Markup {
     html! {
         @let entry_type = entry.entry_type.clone();
@@ -646,7 +660,7 @@ fn entry_row(
             }
             @if let Some(conf) = actions_conf {
                 td.actions-cell {
-                    (rm_form(conf.rm_route, &entry.link))
+                    (rm_form(conf.rm_route, &entry.link, route_prefix))
                 }
             }
         }
@@ -927,15 +941,9 @@ fn page_header(
 
                         document.querySelector('input[type="file"]').addEventListener('change', async (e) => {
                           const file = e.target.files[0];
-                          const hash = await hashFile(file);
                         });
 
                         async function get256FileHash(file) {
-                          if (!crypto.subtle) {
-                            // `crypto.subtle` is not available in nonsecure context (e.g. non-HTTPS LAN).
-                            // See https://developer.mozilla.org/en-US/docs/Web/API/Crypto/subtle
-                            return "";
-                          }
                           const arrayBuffer = await file.arrayBuffer();
                           const hashBuffer = await crypto.subtle.digest('SHA-256', arrayBuffer);
                           const hashArray = Array.from(new Uint8Array(hashBuffer));
@@ -989,7 +997,12 @@ fn page_header(
 
                                 // Upload the single file in a multipart request.
                                 return new Promise(async (resolve, reject) => {
-                                    const fileHash = await get256FileHash(file);
+                                    // File hash calculation may fail at times:
+                                    //   1. `crypto.subtle` is not available in nonsecure context (e.g. non-HTTPS LAN).
+                                    //      See https://developer.mozilla.org/en-US/docs/Web/API/Crypto/subtle
+                                    //   2. For files larger than 2GB, Firefox will refuse to calculate the SHA-256 value,
+                                    //      while Chrome will refuse to create a ArrayBuffer (#1541).
+                                    const fileHash = await get256FileHash(file).catch(() => "");
                                     const xhr = new XMLHttpRequest();
                                     const formData = new FormData();
                                     formData.append('file', file);
@@ -1077,6 +1090,54 @@ fn page_header(
                                 })
                             }
                         }
+
+                        // Bind pastebin submission to create a text/plain blob which is injected
+                        // into the upload input then submitted. A title is automatically generated
+                        // if none is given.
+                        const fileUploadForm = document.querySelector('#file_submit');
+                        const fileUploadInput = document.querySelector('#file_submit input[type=file]');
+                        const pastebinForm = document.querySelector('form#pastebin');
+                        const pastebinFilename = pastebinForm.querySelector('input[name=paste_filename]');
+                        const pastebinContent = pastebinForm.querySelector('textarea');
+                        pastebinContent.addEventListener('keydown', (event) => {
+                            // common convenience of ctrl-enter to submit
+                            if (event.key === 'Enter' && (event.ctrlKey || event.metaKey)) {
+                                event.preventDefault();
+                                event.target.form.requestSubmit();
+                            }
+                        });
+
+                        pastebinForm.addEventListener('submit', (event) => {
+                            // The pastebin form is "dead" and should not cause any page-submit
+                            // events. We capture the pastebin form content, convert it into a
+                            // in-memory blob, then pass that blob to the regular fileUpload form
+                            // for submission, as if a user and selected a real file.
+                            event.preventDefault();
+                            const text = pastebinContent.value;
+                            const title = ((inputValue) => {
+                                const title = inputValue.trim();
+                                if (title.length === 0) {
+                                    const suffix = crypto.randomUUID().substring(0,6);
+                                    return `paste-${suffix}.txt`;
+                                } else {
+                                    // use given extension if one is present, otherwise make it
+                                    // .txt. We're quite liberal in what we consider an extension,
+                                    // any number of alpha-numeric after a dot.
+                                    if (/\.[0-9a-z]+$/i.test(title)) {
+                                        return title;
+                                    } else {
+                                        return `${title}.txt`;
+                                    }
+                                }
+                            })(pastebinFilename.value);
+                            // Package text as a file and submit
+                            const blob = new Blob([text], {type: 'text/plain'});
+                            const file = new File([blob], title, {type: 'text/plain'});
+                            const container = new DataTransfer();
+                            container.items.add(file);
+                            fileUploadInput.files = container.files;
+                            fileUploadForm.submit();
+                        });
                     }
                     "#))
                 }
@@ -1223,4 +1284,21 @@ mod tests {
         let expected = to_html("-H -P 'github.com' 'https://github.com");
         assert_eq!(to_be_tested, expected);
     }
+
+    #[test]
+    fn test_rm_form_strips_prefix() {
+        let rm_route = "/rm";
+        let prefix = "/prefix";
+        let encoded_path = "/prefix/some/path/file.txt";
+
+        let html = rm_form(rm_route, encoded_path, prefix);
+        let expected_action = r#"action="/rm?path=/some/path/file.txt""#;
+
+        assert!(
+            html.0.contains(expected_action),
+            "Actual HTML: {}\nExpected to contain: {}",
+            html.0,
+            expected_action
+        )
+    }
 }
