@@ -1,6 +1,6 @@
 use std::process::Command;
 
-use assert_cmd::prelude::*;
+use assert_cmd::{cargo, prelude::*};
 use clap::{ValueEnum, crate_name, crate_version};
 use clap_complete::Shell;
 
@@ -11,7 +11,7 @@ use crate::fixtures::Error;
 #[test]
 /// Show help and exit.
 fn help_shows() -> Result<(), Error> {
-    Command::cargo_bin("miniserve")?
+    Command::new(cargo::cargo_bin!("miniserve"))
         .arg("-h")
         .assert()
         .success();
@@ -22,7 +22,7 @@ fn help_shows() -> Result<(), Error> {
 #[test]
 /// Show version and exit.
 fn version_shows() -> Result<(), Error> {
-    Command::cargo_bin("miniserve")?
+    Command::new(cargo::cargo_bin!("miniserve"))
         .arg("-V")
         .assert()
         .success()
@@ -35,7 +35,7 @@ fn version_shows() -> Result<(), Error> {
 /// Print completions and exit.
 fn print_completions() -> Result<(), Error> {
     for shell in Shell::value_variants() {
-        Command::cargo_bin("miniserve")?
+        Command::new(cargo::cargo_bin!("miniserve"))
             .arg("--print-completions")
             .arg(shell.to_string())
             .assert()
@@ -48,7 +48,7 @@ fn print_completions() -> Result<(), Error> {
 #[test]
 /// Print completions rejects invalid shells.
 fn print_completions_invalid_shell() -> Result<(), Error> {
-    Command::cargo_bin("miniserve")?
+    Command::new(cargo::cargo_bin!("miniserve"))
         .arg("--print-completions")
         .arg("fakeshell")
         .assert()
